@@ -1,11 +1,12 @@
 
 tree.detection.single.scan <- function(data, single.tree = NULL,
                                        dbh.min = 4, dbh.max = 200, h.min = 1.3,
-                                       ncr.threshold = 0.1, tls.resolution = list(),
-                                       stem.section = NULL, stem.range = NULL, breaks = NULL,
+                                       ncr.threshold = 0.1, tls.resolution = list(), tls.precision = NULL,
+                                       density.reduction = 2,
+                                       stem.section = c(0.7, 3.5), stem.range = NULL, breaks = NULL,
                                        slice = 0.1, understory = NULL, bark.roughness = 1,
                                        den.type = 1, d.top = NULL,
-                                       plot.attributes = NULL,
+                                       plot.attributes = NULL, plot = TRUE,
                                        save.result = TRUE, dir.result = NULL){
 
 
@@ -72,6 +73,23 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
     woody <- woody[!is.na(woody$x) & !is.na(woody$y) & !is.na(woody$z), ]} else {woody <- data}
 
 
+
+  if(!is.null(data$intensity) & suppressWarnings(mean(data$intensity, na.rm = T)) > 0 & is.null(data$GLA)){
+
+    woody <- data[data$intensity > mean(data$intensity, na.rm = T), ]
+
+  }
+
+
+
+  if(!is.null(data$intensity) & suppressWarnings(mean(data$intensity, na.rm = T)) > 0 & !is.null(data$GLA)){
+
+    woody <- woody[woody$intensity > mean(woody$intensity, na.rm = T), ]
+
+  }
+
+
+
   # Statistical filtering of a point cloud
   # Implements the Statistical Outliers Removal (SOR)
 
@@ -80,19 +98,16 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
   woody <- woody[, c("x", "y", "z")]
   woody <- VoxR::filter_noise(data = data.table::setDT(woody), store_noise = TRUE, message = FALSE)
 
-  noise <- woody[woody$Noise == 2, ]
-
+  # noise <- woody[woody$Noise == 2, ]
   woody <- woody[woody$Noise == 1, ]
 
   woody <- merge(data, woody[, c("x", "y", "z")], by = c("x", "y", "z"), all = FALSE)
-  noise <- merge(data, noise, by = c("x", "y", "z"), all = FALSE)
+  # noise <- merge(data, noise, by = c("x", "y", "z"), all = FALSE)
 
 
   # Detection of stem part without shrub vegetation and crown
 
-  message("Detecting tree stem axes")
-
-  stem <- woody[woody$prob.selec == 1, ]
+  stem <- woody
 
   # Defining the vertical section in which trees are detected
 
@@ -108,10 +123,28 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
   }
 
+  message("Retaining points with high verticality values")
 
-  stem <- VoxR::vox(stem[, c("x", "y", "z")], res = 0.03)
+  # stem <- .ver.remove.slice.double(stem)
+  # stem$ver <- ifelse(is.na(stem$ver), stats::runif(length(stem$ver[is.na(stem$ver)])), stem$ver)
+  # stem$ver <- 1 - stem$ver
+  # stem$prob.ver <- stats::runif(nrow(stem))
+  # stem <- stem[stem$ver > stem$prob.ver, ]
+  #
+  # woody <- woody[woody$z <= stem.section[1] | woody$z >= stem.section[2], ]
+  # woody <- rbind(woody, stem[, 1:ncol(woody)])
+
+
+  message("Detecting tree stem axes")
+
+  stem <- stem[stem$prob.selec == 1, ]
+
+  if(is.null(tls.precision)){
+    stem <- VoxR::vox(stem[, c("x", "y", "z")], res = 0.03)} else {
+      stem <- VoxR::vox(stem[, c("x", "y", "z")], res = tls.precision)}
   stem <- stem[, c("x", "y", "z", "npts")]
 
+
   stem <- VoxR::project_voxels(stem)
   # plot(stem$x, stem$y, asp = 1, col = "grey")
 
@@ -120,8 +153,13 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
   stem.2 <- NULL
 
-  stem <- stem[stem$npts > mean(stem$npts) & stem$ratio > mean(stem$ratio) & stem$nvox > mean(stem$nvox), ]
-  # points(stem$x, stem$y, col = "green")
+
+  if(density.reduction == 1)
+    stem <- stem[stem$npts > mean(stem$npts) & stem$nvox > mean(stem$nvox), ]
+
+  if(density.reduction == 2)
+    stem <- stem[stem$npts > mean(stem$npts) & stem$nvox > mean(stem$nvox) & stem$ratio > mean(stem$ratio), ]
+
 
 
   if(!is.null(understory)){
@@ -134,8 +172,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
     }
 
 
-  buf <- sf::st_as_sf(stem, coords = c("x","y"))
-  # sf::st_crs(buf) <- "+proj=utm +zone=19 +ellps=GRS80 +datum=NAD83 +unit=m"
+  buf <- sf::st_as_sf(data.frame(stem), coords = c("x","y"))
   buf <- sf::st_buffer(buf, max(.dbh.min, 0.5))
   buf <- sf::st_cast(sf::st_union(buf), "POLYGON")
 
@@ -185,15 +222,16 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
 
 
-  woody.2 <- sf::st_intersects(buf, sf::st_as_sf(woody, coords = c("x","y")))
+  woody.2 <- sf::st_intersects(buf, sf::st_as_sf(data.frame(woody), coords = c("x","y")))
   woody.2 <- data.table::setDT(as.data.frame(woody.2))
   colnames(woody.2) <- c("tree", "code")
   woody$code <- as.numeric(row.names(woody))
   woody <- merge(woody, woody.2, by = "code", all = FALSE)
   # woody <- subset(woody, select = -code)
-  woody <- woody[, 2:ncol(woody)]
+  woody <- woody[, !(names(woody) %in% c("code"))]
+
+  # woody <- woody[, 2:ncol(woody)]
 
-  # woody <- woody[woody$tree %in% eje$tree, ]
   woody <- woody[!is.na(woody$tree), ]
 
 
@@ -278,10 +316,12 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     .cut <- .ncr.remove.slice.double(.cut)
 
-    .cut <- .cut[which(.cut$ncr < ncr.threshold | is.na(.cut$ncr)), , drop = FALSE]
+    # .cut <- .cut[which(.cut$ncr < ncr.threshold | is.na(.cut$ncr)), , drop = FALSE]
+    .cut <- woody[woody$z > (cuts-2*slice-0.05) & woody$z < cuts, , drop = FALSE]
 
     # Restrict to slice corresponding to cuts m +/- 5 cm
-    .cut <- .cut[which(.cut$z > (cuts-slice) & .cut$z < (cuts+slice)), , drop = FALSE]
+    # .cut <- .cut[which(.cut$z > (cuts-slice) & .cut$z < (cuts+slice)), , drop = FALSE]
+    .cut <- .cut[.cut$z > (cuts-2*slice) & .cut$z < cuts, , drop = FALSE]
 
     # DBSCAN parameters
     .eps <- 2 * (tan(.alpha.h / 2) * (max(.cut$r) / cos(mean(.cut$slope, na.rm = TRUE))) * 2)
@@ -300,12 +340,30 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     .cut$sec <- cuts
 
-    .filter <- do.call(rbind, lapply(split(.cut, .cut$cluster), .sections.single.scan, .cut = .cut,
+    if (interactive()) {
+    # Create a cluster
+
+    cl <- parallel::makeCluster(parallel::detectCores() - 1)
+
+    .filter <- do.call(rbind, parallel::clusterApply(cl, split(.cut, .cut$cluster), .sections.single.scan, .cut = .cut,
                                      .alpha.v = .alpha.v, .alpha.h = .alpha.h,
                                      .dbh.min = .dbh.min, .dbh.max = .dbh.max,
                                      slice = slice * 2, bark.roughness = bark.roughness,
                                      x.center = x.center, y.center = y.center))
 
+    parallel::stopCluster(cl)
+
+    } else {
+
+
+      .filter <- do.call(rbind, lapply(split(.cut, .cut$cluster), .sections.single.scan, .cut = .cut,
+                                       .alpha.v = .alpha.v, .alpha.h = .alpha.h,
+                                       .dbh.min = .dbh.min, .dbh.max = .dbh.max,
+                                       slice = slice * 2, bark.roughness = bark.roughness,
+                                       x.center = x.center, y.center = y.center))
+
+    }
+
     .filteraux<-rbind(.filteraux, .filter)
 
 
@@ -456,6 +514,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
     # section from which radius is estimated, number of used sections (or cuts)
     # will not be important. An estimated radius will always exist
     .filter$dif <- 1.3 - .filter$sec
+    .filter$slope <- ifelse(.filter$dif != 0 & is.na(.filter$slope), mean(.filter$slope, na.rm = TRUE), .filter$slope)
     .filter$radio.est <- ifelse(.filter$dif == 0, .filter$radius, .filter$radius + .filter$slope * .filter$dif)
     .filter <- .filter[!is.na(.filter$radio.est), ]
 
@@ -467,7 +526,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     top.lim <- max(1.3, abs(max(stem.section) - 1.3))
 
-    .radio.est <- data.frame(radio.est = as.numeric())
+    .radio.est <- data.frame(tree = as.numeric(), radio.est = as.numeric())
 
     for (i in unique(.filter$tree)) {
 
@@ -493,7 +552,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
       }
 
-      .out <- data.frame(radio.est = mean(.dat$radio.est, na.rm = TRUE))
+      .out <- data.frame(tree = i, radio.est = mean(.dat$radio.est, na.rm = TRUE))
       .radio.est <- rbind(.radio.est, .out)
 
     }
@@ -561,6 +620,10 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     }
 
+    # .filteraux <- subset(.filteraux, select = -radio.est)
+    .filteraux <- .filteraux[, !(names(.filteraux) %in% c("radio.est"))]
+    .filteraux <- merge(.filteraux, .radio.est, by = "tree")
+
     # Dendrometric variables
     .tree <- data.frame(tree = tapply(.filteraux$tree, .filteraux$tree, mean, na.rm = TRUE),
                         filter = tapply(.filteraux$filter, .filteraux$tree, mean, na.rm = TRUE),
@@ -580,7 +643,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
                         theta = tapply(.filteraux$center.theta, .filteraux$tree, mean,na.rm = TRUE),
 
                         horizontal.distance = tapply(.filteraux$center.rho, .filteraux$tree, mean, na.rm = TRUE), # repeated line
-                        radius = .radio.est$radio.est,
+                        radius = tapply(.filteraux$radio.est, .filteraux$tree, mean, na.rm = TRUE),
 
                         partial.occlusion = tapply(.filteraux$arc.circ, .filteraux$tree, min, na.rm = TRUE),
 
@@ -685,9 +748,13 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
         .filt <- .tree[.tree$tree == i, ]
         .filteraux <- .tree[.tree$tree != i, ]
 
-        if(nrow(.filteraux) < 1)
+        if(nrow(.filteraux) < 1){
+
+          .tree.2 <- rbind(.tree.2, .filt)
           next
 
+        }
+
         .filteraux$dist <- sqrt((.filteraux$x - .filt$x) ^ 2 + (.filteraux$y - .filt$y) ^ 2) - .filteraux$radius - .filt$radius
         .filteraux$rho.dist <- abs(.filteraux$rho - .filt$rho) - .filteraux$radius - .filt$radius
         .filteraux$phi.dist <- abs(.filteraux$phi - .filt$phi)
@@ -700,7 +767,10 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
           .filteraux <- rbind(.filt, .filteraux[ , 1:(ncol(.filteraux)-3)])
 
-          .filt <- .filteraux[.filteraux$filter == max(.filteraux$filter) & .filteraux$sec.max ==  min(.filteraux$sec.max), ]
+          .filt <- .filteraux[.filteraux$filter == max(.filteraux$filter), ]
+
+          if(nrow(.filt) > 1)
+            .filt <- .filteraux[.filteraux$sec.max ==  min(.filteraux$sec.max), ]
 
 
           if(nrow(.filt) > 1)
@@ -773,6 +843,11 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
     data <- data[data$prob.selec == 1, ]
     data <- data[, c("id", "file", "x", "y", "z", "rho")]
 
+
+    if(!is.null(plot))
+      plotTree <- suppressMessages(lidR::plot(lidR::LAS(data[, c("x","y","z")])))
+
+
     # If only one tree is detected, Voronoi tessellation is not working
 
     if(nrow(.tree) == 1){
@@ -904,7 +979,7 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
 
     straightness <- do.call(rbind, lapply(split(.stem, .stem$tree), .straightness, stem.range = stem.range))
 
-    .tree <- merge(.tree, straightness, by = "tree")
+    .tree <- merge(.tree, straightness, by = "tree", all.x = TRUE)
 
 
     utils::write.csv(.stem,
@@ -972,11 +1047,63 @@ tree.detection.single.scan <- function(data, single.tree = NULL,
                      row.names = FALSE)
   }
 
-  if(isTRUE(save.result)){
+  # if(isTRUE(save.result)){
+  #
+  #   .data.red <- noise[which(noise$prob.selec == 1), , drop = FALSE]
+  #
+  #   vroom::vroom_write(.data.red, path = file.path(dir.result, paste("noise_", .data.red$file[1], sep = "")), delim = ",", progress = FALSE)
+  #
+  # }
 
-    .data.red <- noise[which(noise$prob.selec == 1), , drop = FALSE]
 
-    vroom::vroom_write(.data.red, path = file.path(dir.result, paste("noise_", .data.red$file[1], sep = "")), delim = ",", progress = FALSE)
+  diameter <- data.frame(tree = as.numeric(),
+                         x = as.numeric(),
+                         y = as.numeric(),
+                         z = as.numeric())
+
+  phi <- seq(from = 0, to = 2*pi, by = 2 * pi / 10000)
+
+
+  for (i in .tree$tree) {
+
+    tree <- rep(i, times = 10001)
+    x <- .tree$x[i] + cos(phi) * ((.tree$dbh[i] / 100) / 2)
+    y <- .tree$y[i] + sin(phi) * ((.tree$dbh[i] / 100) / 2)
+    z <- stats::runif(10001, 1.2, 1.4)
+
+    .diameter <- data.frame(tree = tree,
+                            x = x, y = y, z = z)
+
+    diameter <- rbind(diameter, .diameter)
+
+  }
+
+
+  if(!is.null(plot)){
+
+    diameter <- data.frame(tree = as.numeric(),
+                           x = as.numeric(),
+                           y = as.numeric(),
+                           z = as.numeric())
+
+    phi <- seq(from = 0, to = 2*pi, by = 2 * pi / 10000)
+
+
+    for (i in .tree$tree) {
+
+      tree <- rep(i, times = 10001)
+      x <- .tree$x[i] + cos(phi) * ((.tree$dbh[i] / 100) / 2)
+      y <- .tree$y[i] + sin(phi) * ((.tree$dbh[i] / 100) / 2)
+      z <- stats::runif(10001, 1.2, 1.4)
+
+      .diameter <- data.frame(tree = tree,
+                              x = x, y = y, z = z)
+
+      diameter <- rbind(diameter, .diameter)
+
+    }
+
+    suppressMessages(lidR::plot(lidR::LAS(diameter[, c("x","y","z")]), add = plotTree, size = 5))
 
   }
 
